!2345678911234567892123456789312345678941234567895123456789612345678971234567898
!        0         0         0         0         0         0         0
!                   
! Module: atthkgen_mod
!
! Authors:  Andy Read - XMM-SSC-MPE
!          
! Description: The task atthkgen produces a periodic attitude HK file for an entire observation.
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module atthkgen_mod

!  use cal
  use oal
  use dal
  use param  
  use errorHandling
  
  implicit none 
  
  private 

  public atthkgen

contains 
  
  !---------------------------------------------------------------------
  subroutine atthkgen
  !---------------------------------------------------------------------

  type (SpacecraftAttitudeType)                 :: att
  type (OdfInfoType)                            :: odfinfos
  type (ProposalInfoType)                       :: proposalinfo
  type(DataSetT)                                :: outhk
  type(TableT)                                  :: hktab
  type(ColumnT)                                 :: ahfra, ahfdec, ahfpa, &
                                                   omra, omdec, ompa, &
                                                   dahfpnt, dompnt, dahfom, time

  character(len=*), parameter                   :: taskname = "atthkgen"
  character(len=256)                            :: history
!  character(len=64)                             :: filter
  character(len=80)                             :: obsid !, expid
 
  integer				        :: nsteps, perc_done, & !nn, 
                                                   n, nahf, nom, nahfom, &
                                                   nahftot, nomtot, nahfomtot

  real(kind=double)                             :: t, begin, end, timestep, &
                                                   tot_ahfra, &  !rapnt, decpnt, 
                                                   tot_ahfdec, tot_ahfpa, &
                                                   tot_omra, tot_omdec, &
                                                   tot_ompa, tot_dahfpnt, &
                                                   tot_dompnt, tot_dahfom, &
                                                   av_ahfra, &
                                                   av_ahfdec, av_ahfpa, &
                                                   av_omra, av_omdec, &
                                                   av_ompa, av_dahfpnt, &
                                                   av_dompnt, av_dahfom, &
                                                   md_ahfra, &
                                                   md_ahfdec, md_ahfpa, &
                                                   md_omra, md_omdec, &
                                                   md_ompa, md_dahfpnt, &
                                                   md_dompnt, md_dahfom
                                                   
  real(kind=double), dimension(:), pointer      :: p_ahfra, p_ahfdec, p_ahfpa, &
                                                   p_omra, p_omdec, p_ompa, &
                                                   p_dahfpnt, p_dompnt, &
                                                   p_dahfom, p_time

  real, parameter                               :: radian = 57.29577951E0

!******************************************************************************
!	START 
!*************************
!--------------------------------------------------------
!	OPEN OAL, get start/end times and nominal RA/dec
!--------------------------------------------------------
    call OAL_odfInfo(odfinfos)
    write(errstr,*) "Obs. runs ",odfinfos%observationStartTime," to ",odfinfos%observationEndTime
    call message(AppMsg,Verbose)
    begin = toTimeTag(odfinfos%observationStartTime)
    end = toTimeTag(odfinfos%observationEndTime)
    write(errstr,*) "Observation start time: ",begin
    call message(AppMsg,Verbose)
    write(errstr,*) "Observation end time: ",end
    call message(AppMsg,Verbose)
    call OAL_proposalInfo(proposalinfo)

!---------------------------------------------------
!	CREATE HK FILE
!---------------------------------------------------
    t = begin
    timestep = realParameter("timestep")
    if (timestep <= 0) then 
      write (errstr,*) "Parameter timestep (",timestep,") should be greater than 0"
      call error("badTimestep")
    endif
    nsteps  = 1 + nint ((end - begin)/timestep)
    write(errstr,*) "Increments in ",nsteps-1,"(+ 1) steps of ",timestep," seconds"
    call message(AppMsg,Verbose)

    outhk = dataSet(stringParameter("atthkset"),CREATE)
    hktab = addTable(outhk,"ATTHK",nsteps)
    time = addColumn(hktab,"TIME",Real64,"sec",comment="time")
    ahfra = addColumn(hktab,"AHFRA",Real64,"degrees",comment="AHF Right Ascension")
    ahfdec = addColumn(hktab,"AHFDEC",Real64,"degrees",comment="AHF Declination")
    ahfpa = addColumn(hktab,"AHFPA",Real64,"degrees",comment="AHF Position Angle")
    omra = addColumn(hktab,"OMRA",Real64,"degrees",comment="OM Right Ascension")
    omdec = addColumn(hktab,"OMDEC",Real64,"degrees",comment="OM Declination")
    ompa = addColumn(hktab,"OMPA",Real64,"degrees",comment="OM Position Angle")
    dahfpnt = addColumn(hktab,"DAHFPNT",Real64,"degrees",comment="Pointing Difference (AHF-PNT)")
    dompnt = addColumn(hktab,"DOMPNT",Real64,"degrees",comment="Pointing Difference (OM-PNT)")
    dahfom = addColumn(hktab,"DAHFOM",Real64,"degrees",comment="Pointing Difference (AHF-OM)")

    p_time => real64Data(time)
    p_ahfra => real64Data(ahfra)
    p_ahfdec => real64Data(ahfdec)
    p_ahfpa => real64Data(ahfpa)
    p_omra => real64Data(omra)
    p_omdec => real64Data(omdec)
    p_ompa => real64Data(ompa)
    p_dahfpnt => real64Data(dahfpnt)
    p_dompnt => real64Data(dompnt)
    p_dahfom => real64Data(dahfom)
!---------------------------------------------------
!	Check nulls
!---------------------------------------------------
    call nullcheck(nullable(ahfra))

!---------------------------------------------------
!	SET UPS BEFORE MAIN LOOP (1)
!---------------------------------------------------
    n = 0
    nahf = 0
    nom = 0
    nahfom = 0
    nahftot = 0
    nomtot = 0
    nahfomtot = 0
    tot_ahfra = 0.0
    tot_ahfdec = 0.0
    tot_ahfpa = 0.0
    tot_omra = 0.0
    tot_omdec = 0.0
    tot_ompa = 0.0
    tot_dahfom = 0.0
    av_ahfra = 400.0
    av_ahfdec = 100.0
    av_ahfpa = 400.0
    av_omra = 400.0
    av_omdec = 100.0
    av_ompa = 400.0
    av_dahfom = 400.0
    md_ahfra = 400.0
    md_ahfdec = 100.0
    md_ahfpa = 400.0
    md_omra = 400.0
    md_omdec = 100.0
    md_ompa = 400.0
    md_dahfom = 400.0
    perc_done=20
!---------------------------------------------------
!	MAIN AHF/OM LOOP (loop 1)
!---------------------------------------------------
    do n = 1, nsteps
      if (n >= perc_done * nsteps /100) then 
        if (perc_done < 100) then
          write(errstr,*) "",perc_done,"% completed of 1st run (AHF/OM)"
        else
          write(errstr,*) perc_done,"% completed of 1st run (AHF/OM)"
        endif
        call message(AppMsg,Sparse)
        perc_done = perc_done + 20
      endif
      p_time(n-1) = t
      !--------------------------------!
      !  Attitude and quality (AHF)    !
      !--------------------------------!
      call OAL_setState(attitudeFromAhf = .TRUE.)	! AHF
      if (.not. OAL_getAttitude(t, att)) then 		! attitude bad
        call setToNull(ahfra,n-1)
        call setToNull(ahfdec,n-1)
        call setToNull(ahfpa,n-1)
        call setToNull(dahfpnt,n-1)
      else						! attitude good
        p_ahfra(n-1) = att%ra * radian
        p_ahfdec(n-1) = att%dec * radian 
        p_ahfpa(n-1) = att%apos * radian
        !--------------------------------!
        !  Averages + Stats (AHF)        !
        !--------------------------------!
        nahf = nahf + 1
        tot_ahfra = tot_ahfra + p_ahfra(n-1)
        tot_ahfdec = tot_ahfdec + p_ahfdec(n-1)
        tot_ahfpa = tot_ahfpa + p_ahfpa(n-1)
      endif
      !--------------------------------!
      !  Attitude and quality (OM)     !
      !--------------------------------!
      call OAL_setState(attitudeFromAhf = .FALSE.)	! OM
      if (.not. OAL_getAttitude(t, att)) then 		! attitude bad
        call setToNull(omra,n-1)
        call setToNull(omdec,n-1)
        call setToNull(ompa,n-1)
        call setToNull(dompnt,n-1)
      else						! attitude good
        p_omra(n-1) = att%ra * radian
        p_omdec(n-1) = att%dec * radian
        p_ompa(n-1) = att%apos * radian
        !--------------------------------!
        !  Averages + Stats (OM)         !
        !--------------------------------!
        nom = nom + 1
        tot_omra = tot_omra + p_omra(n-1)
        tot_omdec = tot_omdec + p_omdec(n-1)
        tot_ompa = tot_ompa + p_ompa(n-1)
      endif
      !--------------------------------!
      !  Difference (AHF - OM)         !
      !--------------------------------!
      if (.not.isNull(dahfpnt,n-1) .and. .not.isNull(dompnt,n-1)) then 
        if ( (p_ahfra(n-1) - p_omra(n-1)) < 1.0e-06 .and. &
             (p_ahfdec(n-1) - p_omdec(n-1)) < 1.0e-06) then 
          p_dahfom(n-1) = 0.0
        else
          p_dahfom(n-1) = (dcos(p_ahfra(n-1)/radian)*dcos(p_ahfdec(n-1)/radian)* &
                         dcos(p_omra(n-1)/radian)*dcos(p_omdec(n-1)/radian)) + &
                        (dsin(p_ahfra(n-1)/radian)*dcos(p_ahfdec(n-1)/radian)* &
                         dsin(p_omra(n-1)/radian)*dcos(p_omdec(n-1)/radian)) + &
                        (dsin(p_ahfdec(n-1)/radian)*dsin(p_omdec(n-1)/radian))
          if (p_dahfom(n-1) > 1.0) p_dahfom(n-1) = 1.0
          if (p_dahfom(n-1) < -1.0) p_dahfom(n-1) = -1.0
          p_dahfom(n-1) = radian * abs(dacos(p_dahfom(n-1)))
        endif
        !--------------------------------!
        !  Averages + Stats (AHF - OM)   !
        !--------------------------------!
        nahfom = nahfom + 1
        tot_dahfom = tot_dahfom + p_dahfom(n-1)
      else
        call setToNull(dahfom,n-1)
      endif
      t = t + timestep
    enddo
    nahftot = nahf
    nomtot = nom
    nahfomtot = nahfom
!---------------------------------------------------
!	END OF AHF/OM LOOP (loop 1)
!---------------------------------------------------
!---------------------------------------------------
!	Calculate AHF and OM averages
!---------------------------------------------------
    if (nahftot >= 0.5) then 
      av_ahfra = tot_ahfra / nahftot
      av_ahfdec = tot_ahfdec / nahftot
      av_ahfpa = tot_ahfpa / nahftot
    else
      av_ahfra = 400.0
      av_ahfdec = 400.0
      av_ahfpa = 400.0
      av_dahfpnt = 400.0
      call warning("BadAhfAttitude","No good AHF attitude found")
    endif   
    if (nomtot >= 0.5) then 
      av_omra = tot_omra / nomtot
      av_omdec = tot_omdec / nomtot
      av_ompa = tot_ompa / nomtot
    else
      av_omra = 400.0
      av_omdec = 400.0 
      av_ompa = 400.0 
      av_dompnt = 400.0 
      call warning("BadOmAttitude","No good OM attitude found")
    endif
    if (nahfomtot >= 0.5) then 
      av_dahfom = tot_dahfom / nahfomtot
    else
      av_dahfom = 400.0
      call warning("BadAhfOmAttitude","No good simultaneous AHF and OM attitude found")
    endif   
!---------------------------------------------------
!	Calculate AHF and OM medians
!---------------------------------------------------
    nahf = 0
    nom = 0
    nahfom = 0
    do n = 1, nsteps
      if (.not.isNull(ahfra,n-1) .and. .not.isNull(ahfdec,n-1) .and. &  
                                 .not.isNull(ahfpa,n-1)) then ! ahf att good
        nahf = nahf + 1
      endif
      if (.not.isNull(omra,n-1) .and. .not.isNull(omdec,n-1) .and. &  
                                    .not.isNull(ompa,n-1)) then   ! om att good
        nom = nom + 1
      endif
      if (.not.isNull(dahfom,n-1)) then                  ! ahf+om attitude good
        nahfom = nahfom + 1
      endif
      if (nahf == nint(real(nahftot)/2)) then 
        md_ahfra = p_ahfra(n-1)
        md_ahfdec = p_ahfdec(n-1)
        md_ahfpa = p_ahfpa(n-1)
      endif
      if (nom == nint(real(nomtot)/2)) then 
        md_omra = p_omra(n-1)
        md_omdec = p_omdec(n-1)
        md_ompa = p_ompa(n-1)
      endif
      if (nahfom == nint(real(nahfomtot)/2)) then 
        md_dahfom = p_dahfom(n-1)
      endif
    enddo
    if (nahf < 0.5) then 
      md_ahfra = 400.0
      md_ahfdec = 400.0 
      md_ahfpa = 400.0 
      md_dahfpnt = 400.0 
    endif
    if (nom < 0.5) then 
      md_omra = 400.0 
      md_omdec = 400.0 
      md_ompa = 400.0 
      md_dompnt = 400.0 
    endif
    if (nahfom < 0.5) then 
        md_dahfom = 400.0
    endif
!---------------------------------------------------
!	END OF AHF/OM MEDIAN LOOP 
!---------------------------------------------------
!write (*,*) md_ahfra,md_ahfdec,md_ahfpa
!---------------------------------------------------
!	SET UPS BEFORE MAIN LOOP (2)
!---------------------------------------------------
    n = 0
    nahf = 0
    nom = 0
    nahfom = 0
    tot_dahfpnt = 0.0
    tot_dompnt = 0.0
    av_dahfpnt = 400.0
    av_dompnt = 400.0
    md_dahfpnt = 400.0
    md_dompnt = 400.0
    perc_done=20
    t = begin
!---------------------------------------------------
!	MAIN D...PNT LOOP (loop 2)
!---------------------------------------------------
    do n = 1, nsteps
      if (n >= perc_done * nsteps /100) then 
        if (perc_done < 100) then
          write(errstr,*) "",perc_done,"% completed of 2nd run (PNT)"
        else
          write(errstr,*) perc_done,"% completed of 2nd run (PNT)"
        endif
        call message(AppMsg,Sparse)
        perc_done = perc_done + 20
      endif
      p_time(n-1) = t
      if (.not. isNull(dahfpnt,n-1)) then 
        !--------------------------------!
        !  Difference (AHF - pnt)        !
        !--------------------------------!
        p_dahfpnt(n-1) = (dcos(p_ahfra(n-1)/radian)*dcos(p_ahfdec(n-1)/radian)*dcos(md_ahfra/radian)* &
                       dcos(md_ahfdec/radian)) + (dsin(p_ahfra(n-1)/radian)*dcos(p_ahfdec(n-1)/radian)* &
                       dsin(md_ahfra/radian)*dcos(md_ahfdec/radian)) + &
                       (dsin(p_ahfdec(n-1)/radian)*dsin(md_ahfdec/radian))
        if (p_dahfpnt(n-1) > 1.0) p_dahfpnt(n-1) = 1.0
        if (p_dahfpnt(n-1) < -1.0) p_dahfpnt(n-1) = -1.0
        p_dahfpnt(n-1) = radian * abs(dacos(p_dahfpnt(n-1)))

        !--------------------------------!
        !  Averages + Stats (AHF)        !
        !--------------------------------!
        nahf = nahf + 1
        tot_dahfpnt = tot_dahfpnt + p_dahfpnt(n-1)
      endif
      if (.not. isNull(dompnt,n-1)) then 
        !--------------------------------!
        !  Difference (OM - pnt)         !
        !--------------------------------!
        p_dompnt(n-1) = (dcos(p_omra(n-1)/radian)*dcos(p_omdec(n-1)/radian)*dcos(md_omra/radian)* &
                       dcos(md_omdec/radian)) + (dsin(p_omra(n-1)/radian)*dcos(p_omdec(n-1)/radian)* &
                       dsin(md_omra/radian)*dcos(md_omdec/radian)) + &
                       (dsin(p_omdec(n-1)/radian)*dsin(md_omdec/radian))
        if (p_dompnt(n-1) > 1.0) p_dompnt(n-1) = 1.0
        if (p_dompnt(n-1) < -1.0) p_dompnt(n-1) = -1.0
        p_dompnt(n-1) = radian * abs(dacos(p_dompnt(n-1)))
        !--------------------------------!
        !  Averages + Stats (OM)         !
        !--------------------------------!
        nom = nom + 1
        tot_dompnt = tot_dompnt + p_dompnt(n-1)
      endif
      t = t + timestep
    enddo
!---------------------------------------------------
!	END OF  LOOP (loop 2)
!---------------------------------------------------
!---------------------------------------------------
!	Calculate D...PNT averages
!---------------------------------------------------
    if (nahftot >= 0.5) then 
      av_dahfpnt = tot_dahfpnt / nahftot
    endif   
    if (nomtot >= 0.5) then 
      av_dompnt = tot_dompnt / nomtot
    endif
!---------------------------------------------------
!	Calculate D...PNT medians
!---------------------------------------------------
    nahf = 0
    nom = 0
    nahfom = 0
    do n = 1, nsteps
      if (.not.isNull(ahfra,n-1) .and. .not.isNull(ahfdec,n-1) .and. &  
                                 .not.isNull(ahfpa,n-1)) then ! ahf att good
        nahf = nahf + 1
      endif
      if (.not.isNull(omra,n-1) .and. .not.isNull(omdec,n-1) .and. &  
                                    .not.isNull(ompa,n-1)) then   ! om att good
        nom = nom + 1
      endif
      if (nahf == nint(real(nahftot)/2)) then 
        md_dahfpnt = p_dahfpnt(n-1)
!do nn = -10,10
!write (*,*) nahf,nn,p_dahfpnt(n-1+nn)
!enddo
      endif
      if (nom == nint(real(nomtot)/2)) then 
        md_dompnt = p_dompnt(n-1)
      endif
    enddo
!---------------------------------------------------
!	END OF D...PNT MEDIAN LOOP 
!---------------------------------------------------

!---------------------------------------------------
!	Write out (and calc D..PNT) averages
!---------------------------------------------------
    if (nahftot >= 0.5 .or. nomtot >= 0.5) then 
      write(errstr,*) "**********Mean Values :**********"
    endif
    call message(AppMsg,Verbose)
    if (nahftot >= 0.5) then 
      av_dahfpnt = tot_dahfpnt / nahftot
      write(errstr,*) "AHF RA  (",nahftot," data points): ", av_ahfra
      call message(AppMsg,Verbose)
      write(errstr,*) "AHF Dec (",nahftot," data points): ", av_ahfdec
      call message(AppMsg,Verbose)
      write(errstr,*) "AHF PA  (",nahftot," data points): ", av_ahfpa
      call message(AppMsg,Verbose)
      write(errstr,*) "(AHF-PNT) pointing diff.(",nahftot," data points): ", av_dahfpnt
      call message(AppMsg,Verbose)
    endif   
    if (nomtot >= 0.5) then 
      av_dompnt = tot_dompnt / nomtot
      write(errstr,*) "OM RA  (",nomtot," data points): ", av_omra
      call message(AppMsg,Verbose)
      write(errstr,*) "OM Dec (",nomtot," data points): ", av_omdec
      call message(AppMsg,Verbose)
      write(errstr,*) "OM PA  (",nomtot," data points): ", av_ompa
      call message(AppMsg,Verbose)
      write(errstr,*) "(OM-PNT) pointing diff. (",nomtot," data points): ", av_dompnt
      call message(AppMsg,Verbose)
    endif
    if (nahfomtot >= 0.5) then 
      write(errstr,*) "(AHF-OM) pointing diff. (",nahfomtot," data points): ", av_dahfom
      call message(AppMsg,Verbose)
    endif   
    write(errstr,*) "**********Median Values :**********"
    if (nahftot > 0.5) then 
      call message(AppMsg,Verbose)
      write(errstr,*) "AHF RA  : ", md_ahfra
      call message(AppMsg,Verbose)
      write(errstr,*) "AHF Dec : ", md_ahfdec
      call message(AppMsg,Verbose)
      write(errstr,*) "AHF PA  : ", md_ahfpa
      call message(AppMsg,Verbose)
      write(errstr,*) "(AHF-PNT) pointing diff. : ", md_dahfpnt
      call message(AppMsg,Verbose)
    endif
    if (nomtot > 0.5) then 
      write(errstr,*) "OM RA  : ", md_omra
      call message(AppMsg,Verbose)
      write(errstr,*) "OM Dec : ", md_omdec
      call message(AppMsg,Verbose)
      write(errstr,*) "OM PA  : ", md_ompa
      call message(AppMsg,Verbose)
      write(errstr,*) "(OM-PNT) pointing diff. : ", md_dompnt
      call message(AppMsg,Verbose)
    endif
    if (nahfomtot > 0.5) then 
      write(errstr,*) "(AHF-OM) pointing diff. : ", md_dahfom
      call message(AppMsg,Verbose)
    endif
!-------------------------------------------------
!	ADD ATTRIBUTES TO PRIMARY HEADER
!-------------------------------------------------
    call setAttribute(outhk,"TELESCOP","XMM","Telescope (mission) name")
!    call setAttribute(outhk,"INSTRUME","EPN","Instrument name")
    call setAttribute(outhk,"DETNAM"," ","PRIME or REDUNDANT")

!    if (OAL_hasIPPV("FILTER_WHEEL_POSITION")) then
!      filter = OAL_getIPPVstring("FILTER_WHEEL_POSITION")
!    else
!      call warning("BadFilter","No filter found")
!      filter = 'UNKNOWN'
!    endif
!    call setAttribute(outhk,"FILTER",filter,"Filter ID")

    if (odfInfos%isSlew) then
      call setAttribute(outhk,"OBS_MODE","SLEW", &
                     "Observation mode (pointing or slew)")
    else
      call setAttribute(outhk,"OBS_MODE","POINTING", &
                     "Observation mode (pointing or slew)") 
    endif
    write(obsid,"(2a)") trim(odfinfos%proposalId),trim(odfinfos%observationId)
    call setAttribute(outhk,"OBS_ID",obsid,"Observation identifier")
!    write(expid,"(i3)") expinfos%number
!    call setAttribute(outfile,"EXP_ID",expid,"Exposure identifier")
!    if (inmode == 0 .or. inmode == 1 .or. inmode == 8) then
!      select case (inmode)
!      case(0)
!        call setAttribute(outhk,"CONTENT","EPIC IMAGING MODE EVENT LIST", &
!                        "Content of file")
!        call setAttribute(outhk,"DATAMODE","IMAGING", &
!                        "Instrument mode (IMAGING, TIMING, FAST, etc.)")
!      case(1)
!        call setAttribute(outhk,"CONTENT","EPIC TIMING MODE EVENT LIST", &
!                        "Content of file")
!        call setAttribute(outhk,"DATAMODE","TIMING", &
!                        "Instrument mode (IMAGING, TIMING, FAST, etc.)")
!      case(8)
!        call setAttribute(outhk,"CONTENT","EPIC BURST MODE EVENT LIST", &
!                        "Content of file")
!        call setAttribute(outhk,"DATAMODE","BURST", &
!                        "Instrument mode (IMAGING, TIMING, FAST, etc.)")
!      end select
!    endif
    call setAttribute(outhk,"ORIGIN","Leicester/SSC","origin of FITS file")
!    call setAttribute(outhk,"OBJECT",trim(proposalinfo%targetName), &
!                            "Name of observed object")
!    call setAttribute(outhk,"EXPOSURE", &
!                             toTimeTag(expInfos%actualEndTime(1:23)) - &
!	                     toTimeTag(expInfos%actualStartTime(1:23)),"s", &
!                            "Exposure")
    call setAttribute(outhk,"DATE-OBS",trim(odfinfos%observationStartTime), &
                            "Date observations were made")
    call setAttribute(outhk,"DATE-END",trim(odfinfos%observationEndTime), &
                            "Date observations ended")
!    call setAttribute(outhk,"OBSERVER",trim(proposalinfo%GoName), &
!                            "Name of PI")

    call setAttribute(outhk,"RA_OBJ",proposalinfo%targetRa,"deg", &
                            "Ra of the observed object")
    call setAttribute(outhk,"DEC_OBJ",proposalinfo%targetDec,"deg", &
                            "Dec of the observed object")
    call setAttribute(outhk,"RA_NOM",proposalinfo%boresightRa,"deg", &
                            "Ra of the observed object")

    call setAttribute(outhk,"DEC_NOM",proposalinfo%boresightDec,"deg", &
                            "Dec of the observed object")
!-------------------------------------------------
!	ADD AVERAGE ATTRIBUTES TO PRIMARY HEADER
!-------------------------------------------------
!write (*,*) "here"
    call setAttribute(outhk,"AAHFRA",av_ahfra,"deg", "Mean AHF RA")
    call setAttribute(outhk,"AAHFDEC",av_ahfdec,"deg", "Mean AHF Dec")
    call setAttribute(outhk,"AAHFPA",av_ahfpa,"deg", "Mean AHF PA")
    call setAttribute(outhk,"AOMRA",av_omra,"deg", "Mean OM RA")
    call setAttribute(outhk,"AOMDEC",av_omdec,"deg", "Mean OM Dec")
    call setAttribute(outhk,"AOMPA",av_ompa,"deg", "Mean OM PA")
    call setAttribute(outhk,"ADAHFPNT",av_dahfpnt,"deg", "Mean (AHF-PNT) pointing difference")
    call setAttribute(outhk,"ADOMPNT",av_dompnt,"deg", "Mean (OM-PNT) pointing difference")
    call setAttribute(outhk,"ADAHFOM",av_dahfom,"deg", "Mean (AHF-OM) pointing difference")

    call setAttribute(outhk,"MAHFRA",md_ahfra,"deg", "Median AHF RA")
    call setAttribute(outhk,"MAHFDEC",md_ahfdec,"deg", "Median AHF Dec")
    call setAttribute(outhk,"MAHFPA",md_ahfpa,"deg", "Median AHF PA")
    call setAttribute(outhk,"MOMRA",md_omra,"deg", "Median OM RA")
    call setAttribute(outhk,"MOMDEC",md_omdec,"deg", "Median OM Dec")
    call setAttribute(outhk,"MOMPA",md_ompa,"deg", "Median OM PA")
    call setAttribute(outhk,"MDAHFPNT",md_dahfpnt,"deg", "Median (AHF-PNT) pointing difference")
    call setAttribute(outhk,"MDOMPNT",md_dompnt,"deg", "Median (OM-PNT) pointing difference")
    call setAttribute(outhk,"MDAHFOM",md_dahfom,"deg", "Median (AHF-OM) pointing difference")

    call setAttribute(outhk,"NATT",nsteps,"", "Number of attitude entries")
    call setAttribute(outhk,"NGAHF",nahf,"", "Number of good AHF attitude entries")
    call setAttribute(outhk,"NGOM",nom,"", "Number of good OM attitude entries")
    call setAttribute(outhk,"NGAHFOM",nahfom,"", "Number of good AHF+OM attitude entries")
!-------------------------------------------------
!       ADD HISTORY (can put more information in)
!-------------------------------------------------
    write (history,'(a,i6)') "Number of attitude entries: ",nsteps
    call addHistory(outhk,history)
    write (history,'(a,i6)') "Number of good AHF attitude entries: ",nahf
    call addHistory(outhk,history)
    write (history,'(a,i6)') "Number of good OM attitude entries: ",nom
    call addHistory(outhk,history)
    write (history,'(a,i6)') "Number of good AHF+OM attitude entries: ",nahfom
    call addHistory(outhk,history)
    write (history,'(a,f7.4)') "Atthkgen timestep (seconds): ",timestep
    call addHistory(outhk,history)
!--------------------------------------------------
!	RELEASE FILE
!--------------------------------------------------
!    call release(time)
!    call release(ahfra)
!    call release(ahfdec)
!    call release(ahfpa)
!    call release(omra)
!    call release(omdec)
!    call release(ompa)
!    call release(dahfpnt)
!    call release(dompnt)
!    call release(dahfom)
!write (*,*) "here"
    call release(outhk)
!write (*,*) "here"
!--------------------------------------------------
!	*** END ***
    end subroutine atthkgen
!--------------------------------------------------
  !---------------------------------------------------------------------
    subroutine nullcheck (thisNullable)

    use dal
    type(NullableT),intent(in)  :: thisNullable

    write(errstr,*)   "Null defined?: ",nullDefined(thisNullable)
    call message(AppMsg,Noisy)

    end subroutine nullcheck
  !---------------------------------------------------------------------
end module atthkgen_mod

